package sun.net;

import java.io.IOException;
import java.io.InputStream;
import java.io.FilterInputStream;

public class TelnetInputStream extends FilterInputStream
{
    boolean stickyCRLF;
    boolean seenCR;
    public boolean binaryMode;
    
    public TelnetInputStream(InputStream in, boolean binaryMode) {
        super(in);
        this.stickyCRLF = false;
        this.seenCR = false;
        this.binaryMode = false;
        this.binaryMode = binaryMode;
    }
    
    public void setStickyCRLF(boolean stickyCRLF) {
        this.stickyCRLF = stickyCRLF;
    }
    
    @Override
    public int read() throws IOException {
        if (this.binaryMode) {
            return super.read();
        }
        if (this.seenCR) {
            this.seenCR = false;
            return 10;
        }
        int read;
        if ((read = super.read()) == 13) {
            switch (super.read()) {
                case 0: {
                    return 13;
                }
                case 10: {
                    if (this.stickyCRLF) {
                        this.seenCR = true;
                        return 13;
                    }
                    return 10;
                }
                default: {
                    throw new TelnetProtocolException("misplaced CR in input");
                }
            }
        }
        else {
            return read;
        }
    }
    
    @Override
    public int read(byte[] array) throws IOException {
        return this.read(array, 0, array.length);
    }
    
    @Override
    public int read(byte[] b, int off, int len) throws IOException {
        if (this.binaryMode) {
            return super.read(b, off, len);
        }
        while (--len >= 0) {
            int read = this.read();
            if (read == -1) {
                break;
            }
            b[off++] = (byte)read;
        }
        return (off > off) ? (off - off) : -1;
    }
}
